﻿using System;
using System.Linq;
using CommonBehaviors.Actions;
using JetBrains.Annotations;
using LazyBoxer.Helpers;
using Styx;
using Styx.CommonBot;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;

namespace LazyBoxer.Behaviors
{
    static class FollowTheLeader
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }

        public static bool BotMovement { get; set; }
        public static bool WaitingHere { get; set; }


        private static WoWPoint _lastDest;
        public static WoWPoint LeaderLastLocation;
        public static bool TryLastPosition;
        private static bool _invalidated = true;

        private static WoWUnit _leader;
        public static string FollowLeaderName = "";
        public static WoWUnit Leader
        {
            get
            {
                if (Setting.Instance.FollowLeader == "[Never]") return null;
                if (Setting.Instance.FollowLeader == "[None]") return null;

                if (_leader != null && _leader.IsValid && _leader.IsAlive && _leader.Distance > 30)
                {
                    LeaderLastLocation = _leader.Location;
                }

                // We've specifically identified a player to follow
                if (!Setting.Instance.FollowLeader.Contains("Never") && !Setting.Instance.FollowLeader.Contains("None") && !Setting.Instance.FollowLeader.Contains("Automatic"))
                { 
                    if (_leader == null) { _invalidated = true; }

                    // If we've moved out of range and we've lost the leader this allows us to move to where we last saw them
                    

                    // If the leader has not changed and they are valid return our old leader
                    if (_leader != null && _leader.IsValid && _leader.IsAlive && _leader.Name == Setting.Instance.FollowLeader)
                    {
                        
                        if (_invalidated)
                        {
                            _leader.OnInvalidate += LeaderOnOnInvalidate;
                            
                            _invalidated = false;
                        }

                        TryLastPosition = false;
                        return _leader;
                    }

                    // Find a suitable leader
                    if (_leader == null || !_leader.IsValid || !_leader.IsAlive || _leader.Name != Setting.Instance.FollowLeader)
                    {
                        WoWUnit unit = ObjectManager.GetObjectsOfTypeFast<WoWPlayer>().FirstOrDefault(u => u.IsAlive && u.Name == Setting.Instance.FollowLeader);
                        if (unit != null)
                        {
                            TreeRoot.StatusText = "Leader choosen by name: " + unit.Name;
                            RaFHelper.SetLeader(unit.Guid);
                            _leader = unit;

                            TryLastPosition = false;
                            return unit;
                        }
                    }
                }

                // RAID only search
                // We're here because we don't have a suitable leader. 
                if (Me.GroupInfo.IsInRaid && (!Setting.Instance.FollowLeader.Contains("Never") && !Setting.Instance.FollowLeader.Contains("None")))
                {
                    // If the leader has not changed and they are valid return our old leader
                    if (_leader != null && _leader.IsValid && _leader.IsAlive && !Timers.AutoExpire("Find a tank", 4000)) return _leader;

                    // Find a suitle leader. Search the Tanks...
                    for (int i = 1; i < Me.GroupInfo.RaidMembers.Count(); i++)
                    {
                        string role = Lua.GetReturnVal<string>(String.Format("return UnitGroupRolesAssigned('raid{0}')", i), 0);
                        if (role != "TANK") continue;

                        WoWUnit possibleLeader = ObjectManager.GetObjectByGuid<WoWPlayer>(Me.GetRaidMemberGuid(i - 1));

                        // Make sure they are valid and alive
                        if (possibleLeader != null && !possibleLeader.IsAlive) continue;
                        if (possibleLeader != null)
                        {
                            _leader = possibleLeader;
                            TryLastPosition = false;
                            RaFHelper.SetLeader(_leader.Guid);
                            TreeRoot.StatusText = "Leader choosen by Raid TANK role: " + possibleLeader.Name;

                            return _leader;
                        }
                        break;
                    }
                }

                // Party only search
                // If we're here then we're not in a RAID
                if ((Me.GroupInfo.IsInParty || Me.GroupInfo.IsInLfgParty) && (!Setting.Instance.FollowLeader.Contains("Never") && !Setting.Instance.FollowLeader.Contains("None")))
                {
                    // If the leader has not changed and they are valid return our old leader
                    if (_leader != null && _leader.IsValid && _leader.IsAlive && !Timers.AutoExpire("Find a tank", 4000)) return _leader;

                    // Search our party members
                    for (int i = 1; i < Me.GroupInfo.PartyMembers.Count(); i++)
                    {
                        string role = Lua.GetReturnVal<string>(String.Format("return UnitGroupRolesAssigned('party{0}')", i), 0);
                        if (role != "TANK") continue;

                        WoWUnit possibleLeader = ObjectManager.GetObjectByGuid<WoWPlayer>(Me.GetRaidMemberGuid(i - 1));

                        // Make sure they are valid and alive
                        if (possibleLeader != null && !possibleLeader.IsAlive) continue;
                        if (possibleLeader != null)
                        {
                            _leader = possibleLeader;
                            TryLastPosition = false;
                            TreeRoot.StatusText = "Leader choosen by Party tank role: " + possibleLeader.Name;
                            RaFHelper.SetLeader(_leader.Guid);
                            return _leader;
                        }
                        break;
                    }
                }


                /*
                try
                {
                    WoWPlayer lastResortLeader = ObjectManager.GetObjectsOfTypeFast<WoWPlayer>().FirstOrDefault(p => p.IsAlive && !p.IsMe);

                    if (lastResortLeader != null)
                    {
                        TreeRoot.StatusText = "Leader choosen by last resort: " + lastResortLeader.Name;
                        _leader = lastResortLeader;
                        RaFHelper.SetLeader(_leader.Guid);
                        TryLastPosition = false;
                        return lastResortLeader;
                    }
                }
                catch (Exception e)
                {
                    //Console.WriteLine(e);
                }
                 */

                return null;
            }

            set
            {
                _leader = value;
                RaFHelper.SetLeader(_leader.Guid);
                FollowLeaderName = _leader.Name;
            }
        }

        private static void LeaderOnOnInvalidate()
        {
            TryLastPosition = true;
            _leader = null;
        }

        public static Composite CreateFollowTheLeaderBehaviorsWithSanityChecks()
        {

            return
                // Follow leader - if we're not doing a lot of other stuff
                new Decorator(ret =>

                    // follow if... CombatAssistOnly OR everything is ok

                    (KeyToggles.IsCombatAssistOnly || (Looting.Lootable == null &&
                    Skinning.SkinnableMob == null &&
                    Mining.OreHarvestable == null &&
                    Herbalism.HerbHarvestable == null &&
                    InteractableObjects.QuestItemHarvestable == null)) &&
                    
                    (Leader != null || TryLastPosition) &&
                    !ForceAttack.ForceAttacking &&
                    !Vendoring.SellingAtVendor &&
                    !Me.HasAura("Eat") && !Me.HasAura("Drink") &&

                    ((Me.Combat && Setting.Instance.CombatFollow.OK()) || !Me.Combat),

                    new PrioritySelector(CreateFollowBehavior()));
        }

        //[UsedImplicitly]
        //private static bool _isInitialized;

        public static int FollowDistance { get { return Setting.Instance.CombatFollow.OK() ? Setting.Instance.CombatFollowDistance : Setting.Instance.FollowDistance; } }

        /// <summary>
        /// Does not apply any sanity checks first! Use spareingly and only if you know what you're doing.
        /// </summary>
        /// <returns></returns>
        public static Composite CreateFollowBehavior()
        {
            return new PrioritySelector(

                // Reasons NOT to follow leader
                new Decorator(ret => Setting.Instance.FollowLeader.Contains("Never") || Setting.Instance.FollowLeader.Contains("None"), new ActionAlwaysSucceed()),
                new Decorator(ret => Misc.IsUserControllingMovement, new ActionAlwaysSucceed()),
                new Decorator(ret => WaitingHere, new ActionAlwaysSucceed()),
                new Decorator(ret => Leader == null && !TryLastPosition, new ActionAlwaysSucceed()),
                new Decorator(ret => Me.IsDead || Me.CurrentHealth <= 1, new ActionAlwaysSucceed()),

                // If we're a ghost and the leader is a ghost follow them
                new Decorator(ret => Me.IsGhost && !Leader.IsGhostVisible, new ActionAlwaysSucceed()),

                // Mount up 
                new Decorator(ret=> Common.NeedToMount(),
                              new PrioritySelector(
                                  
                                  new Action(context => { TreeRoot.StatusText = "Mounting up..."; return RunStatus.Failure; }),

                                  // Stop moving
                                  new Decorator(ret=> Me.IsMoving, new Sequence(
                                          Movement.CreateEnsureMovementStoppedBehavior(),
                                          new Wait(2, ret=> !Me.IsMoving, new ActionAlwaysSucceed())
                                          )),


                                          // Mount up
                                  new Decorator(ret=> !Me.IsMoving, Common.MountUp())
                                  //new Wait(5, ret => Me.IsCasting, new ActionAlwaysSucceed())
                                      
                                  )
                    ),

                // Mount up and Dismount as required
                //new Decorator(ret => Common.NeedToMount(), Common.MountUp()),
                new Decorator(ret => Common.NeedToDismount(), Common.Dismount()),//new Action(delegate { Common.WaitForDismount(); })),

                new Decorator(ret => Leader == null && TryLastPosition,
                              new Action(context =>
                                  {
                                      //Log.Info("======== let move to our leaders last location and see if we can find them");
                                      if (Me.Location.Distance(LeaderLastLocation) <= FollowDistance)
                                      {
                                          TryLastPosition = false;
                                          return RunStatus.Failure;
                                      }

                                      //Log.Info("======== lets get moving then....");
                                      WoWPoint ftLocation = LeaderLastLocation;
                                      if (ftLocation.DistanceSqr(_lastDest) > 1 || !Me.IsMoving)
                                      {
                                          _lastDest = ftLocation;


                                          WoWPoint newPoint;

                                          if (Setting.Instance.CloseEnoughLocation.OK())
                                          {
                                              newPoint = WoWMovement.CalculatePointFrom(ftLocation.LocationNearHere(10), (float)0.85 * FollowDistance);
                                          }
                                          else
                                          {
                                              newPoint = WoWMovement.CalculatePointFrom(ftLocation, (float)0.85 * FollowDistance);
                                          }

                                          newPoint = WoWMovement.CalculatePointFrom(ftLocation, (float)0.85 * FollowDistance);

                                          if ((Me.Mounted && Me.IsFlying) || Me.IsFlying || Me.IsSwimming)
                                          {
                                              TreeRoot.StatusText = "Moving to leader's last known location";
                                              Flightor.MoveTo(newPoint);
                                          }
                                          else
                                          {
                                              TreeRoot.StatusText = "Moving to leader's last known location";
                                              Navigator.MoveTo(newPoint);
                                          }

                                          BotMovement = true;
                                      }

                                      return RunStatus.Success;
                              })
                    ),

                // Finally just move to our follow target
                new Decorator(ret => (Leader != null && Leader.Distance > FollowDistance || (Leader != null && !Leader.InLineOfSpellSight)) && (Navigator.CanNavigateFully(Me.WorldLocation, Leader.WorldLocation)), //|| !Me.IsFlying || !Me.IsSwimming),
                    new Action(context =>
                    {
                        WoWPoint ftLocation = Leader.WorldLocation;
                        if (ftLocation.DistanceSqr(_lastDest) > 1 || !Me.IsMoving)
                        {
                            _lastDest = ftLocation;
                            WoWPoint newPoint = WoWMovement.CalculatePointFrom(ftLocation, (float)0.85 * FollowDistance);

                            if ((Me.Mounted && Me.IsFlying) || Me.IsFlying || Me.IsSwimming && (Leader.IsFlying || Leader.IsSwimming))
                            {
                                TreeRoot.StatusText = "Flying / Swimming to our leader";
                                Flightor.MoveTo(newPoint);
                            }
                            else
                            {
                                TreeRoot.StatusText = "Following our leader";

                                Navigator.MoveTo(newPoint);
                            }

                            BotMovement = true;
                        }

                        return RunStatus.Success;
                    })),



                    // Stop moving 
                new Decorator(ret => Me.IsMoving && BotMovement && !Misc.IsUserControllingMovement,
                    new Sequence(
                        new Action(context => BotMovement = false),
                        Movement.CreateEnsureMovementStoppedBehavior(),
                        new WaitContinue(1, ret=> !Me.IsMoving, new ActionAlwaysSucceed()),
                        new Action(context => TreeRoot.StatusText = "")
                        
                        )
                    ),

                    
                    // clear our bot movement flag
                new Decorator(ret => !Me.IsMoving && BotMovement && !Misc.IsUserControllingMovement,
                    new Action(context => BotMovement = false)
                    )

                // Do we need to stop moving?
                /*
                new Decorator(ret => Me.IsMoving && BotMovement,
                    new Action(delegate
                    {
                        BotMovement = false;
                        while (Common.IsGameStable() && Me.IsMoving)
                        {
                            TreeRoot.StatusText = "";
                            WoWMovement.MoveStop();
                            if (Me.IsMoving) { StyxWoW.SleepForLagDuration(); }
                        }

                        return RunStatus.Success;
                    }))
                 */



                );
        }
    }
}
