﻿using System;
using System.Linq;
using CommonBehaviors.Actions;
using LazyBoxer.Helpers;
using Styx;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.CommonBot.Routines;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;

namespace LazyBoxer.Behaviors
{
    static class Herbalism
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }

        public static Composite CreateHerbHarvesting()
        {
            return
                new PrioritySelector(
                    // Harvest Herbs
                    new Decorator(ret => Setting.Instance.InteractHerbalism.OK() && !Misc.IsUserControllingMovement && !Me.IsCasting && !Me.IsChanneling && !Me.Combat && !Me.Looting && HerbHarvestable != null,
                        new PrioritySelector(

                            // Don't spam the log too many times
                            new Throttle(5, new Action(c => { Log.Info("Herb found: " + HerbHarvestable.Name); return RunStatus.Failure; })),
                            
                            // Hostile mob in our path and we've flagged it for death!
                            //new Decorator(ret => Me.GotTarget && BotPoi.Current.Type == PoiType.Kill && Common.HostileMobInOurPath(HerbHarvestable.Distance + 7) != null && Common.HostileMobInOurPath(HerbHarvestable.Distance + 7).Distance < 30,
                            new Decorator(ret => Me.GotTarget && HerbHarvestable != null && BotPoi.Current.Type == PoiType.Kill && Common.HostileMobInMyPath(Me.Location, HerbHarvestable.Location, HerbHarvestable.Distance + 7) != null && Common.HostileMobInMyPath(Me.Location, HerbHarvestable.Location, HerbHarvestable.Distance + 7).Distance < 30,
                                new Sequence(

                                    // Stop moving
                                    new DecoratorContinue(ret => Me.IsMoving,Movement.CreateEnsureMovementStoppedBehavior()),
                                    new WaitContinue(2, ret => !Me.IsMoving, new ActionAlwaysSucceed()),

                                    // Clear navigator so we don't jump
                                    new Action(context => Navigator.Clear()),

                                    // Call the CC pull logic      
                                    new Decorator(ret => RoutineManager.Current.PullBehavior != null,RoutineManager.Current.PullBehavior),

                                    // Call the CC combat logic as some CCs may not have pull logic 
                                    new Decorator(ret => RoutineManager.Current.PullBehavior != null,RoutineManager.Current.CombatBehavior)
                                    )),

                            // Attack mobs in front of us if we're moving to harvest
                            // If we have a hostile mob in our path mark it for death
                            new Decorator(ret => Me.IsMoving && HerbHarvestable!= null && Common.HostileMobInMyPath(Me.Location, HerbHarvestable.Location, HerbHarvestable.Distance + 7) != null && !Me.IsCasting && HerbHarvestable.Distance > HerbHarvestable.InteractRange * 0.85f,
                                new Throttle(1,
                                             new Sequence(
                                                 new Action(context =>
                                                     {
                                                         WoWUnit target = Common.HostileMobInMyPath(Me.Location, HerbHarvestable .Location, HerbHarvestable .Distance + 7);
                                                         Log.Info(string.Format("Hostile mob in our path, {0}, " + "we need to kill it", target.Name));
                                                         target.Target();
                                                         BotPoi.Current = new BotPoi(target, PoiType.Kill);
                                                     })
                                                 )
                                    )),

                            // Move closer to the object
                            new Decorator(ret => !Me.Combat && HerbHarvestable != null && HerbHarvestable.Distance > HerbHarvestable.InteractRange * 0.85f, Movement.CreateMoveToLocationBehavior(ret => HerbHarvestable.WorldLocation, true, ret => HerbHarvestable.InteractRange * 0.55f)),


                            // if we're moving then and we're in interact range then stop
                            new Decorator(ret => HerbHarvestable.Distance <= HerbHarvestable.InteractRange * 0.85f && Me.IsMoving,
                                new Sequence(
                                    Movement.CreateEnsureMovementStoppedBehavior(),

                                    // wait up to 2 seconds for us to register as stopped moving. WoW and HB tend to see this differently
                                    new Wait(2, ret => !Me.IsMoving, new ActionAlwaysSucceed())
                                    )
                                ),



                            // If we're close enough the interact with the objecb
                            new Decorator(ret => HerbHarvestable.Distance <= HerbHarvestable.InteractRange*0.99f && !Me.IsMoving,
                                new Sequence(

                                    new Action(c => TreeRoot.StatusText = string.Format("Herbalising {0}", HerbHarvestable.Name)),

                                    // Finally interact with it
                                    new Action(c =>{_lootableGameObject = HerbHarvestable; try { HerbHarvestable.Interact(); } catch (Exception e) {  }
                                    }),

                                    // wait for casting to start, max 2 seconds
                                    new Wait(2, ret => Me.IsCasting, new ActionAlwaysSucceed()),

                                    // clear all navigation as we tend to jump after we've looted
                                    new Action(c =>{Navigator.Clear();return RunStatus.Success;}),

                                    // wait while we are casting. Max 10 seconds
                                    new WaitContinue(10, ret => !Me.IsCasting, new ActionAlwaysSucceed()),

                                    // Casting has finished, but wait 2 more seconds for looting to occur
                                    new WaitContinue(1, ret => false, new ActionAlwaysSucceed()),
                                    new Wait(1, ret => Me.Looting, new ActionAlwaysSucceed()),
                                    new WaitContinue(5, ret => !Me.Looting, new ActionAlwaysSucceed()),


                                    // Black list so we don't try to loot again for min 1 minute
                                    new Action(c =>Blacklist.Add(_lootableGameObject.Guid, BlacklistFlags.Loot,TimeSpan.FromMinutes(2))),
                                    new Action(c => { _lootableGameObject = null; }),

                                    // clear variable
                                    // clear all navigation as we tend to jump after we've looted
                                    new Action(c =>{Navigator.Clear();return RunStatus.Success;})
                                    ))

                            )
                        )




                    );
        }


        private static WoWGameObject _herbToHarvest = null;
        public static WoWGameObject HerbHarvestable
        {
            get
            {
                if (Setting.Instance.InteractHerbalism.OK())
                {

                    if (Units.IsObjectValid<WoWGameObject>(_herbToHarvest)) return _herbToHarvest;

                    using (StyxWoW.Memory.AcquireFrame())
                    {
                        _herbToHarvest = ObjectManager.GetObjectsOfTypeFast<WoWGameObject>()
                                                      .Where( o =>
                                                              o.IsHerb && o.CanLoot &&
                                                              !Misc.IsPlayerCompeting(o) &&
                                                              !Blacklist.Contains(o.Guid, BlacklistFlags.Node) &&
                                                              o.WorldLocation.Distance(StyxWoW.Me.Location) <=
                                                              Misc.InteractRange)
                                                      .OrderBy(o => o.DistanceSqr)
                                                      .FirstOrDefault();
                    }


                    return _herbToHarvest;
                }

                return null;
            }
        }

        private static WoWGameObject _lootableGameObject = null;

        
    }
}
