﻿using System;
using System.Linq;
using CommonBehaviors.Actions;
using LazyBoxer.Helpers;
using Styx;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.CommonBot.Routines;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;

namespace LazyBoxer.Behaviors
{
    static class InteractableObjects
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }


        public static Composite CreateQuestObjectInteraction()
        {
            return

                new Decorator(ret => Setting.Instance.QuestObjectInteract.OK() && !Misc.IsUserControllingMovement && !Me.IsCasting && !Me.IsChanneling && !Me.Combat && QuestItemHarvestable != null,
                        new PrioritySelector(

                            // Hostile mob in our path and we've flagged it for death!
                            new Decorator(ret => Behaviors.Common.HostileMobInOurPath(QuestItemHarvestable.Distance + 7) != null && Behaviors.Common.HostileMobInOurPath(QuestItemHarvestable.Distance + 5).Distance < 30 && Me.GotTarget && BotPoi.Current.Type == PoiType.Kill,
                                          new Sequence(

                                              // Stop moving
                                              new DecoratorContinue(ret => Me.IsMoving, Movement.CreateEnsureMovementStoppedBehavior()),
                                              new WaitContinue(2, ret => !Me.IsMoving, new ActionAlwaysSucceed()),

                                              // Clear navigator so we don't jump
                                              new Action(context => Navigator.Clear()),

                                              // Call the CC pull logic      
                                              new Decorator(ret => RoutineManager.Current.PullBehavior != null, RoutineManager.Current.PullBehavior),

                                              // Call the CC combat logic as some CCs may not have pull logic 
                                              new Decorator(ret => RoutineManager.Current.PullBehavior != null, RoutineManager.Current.CombatBehavior)
                                          )),

                            new Action(c =>{Log.Info("Interactable object found: " + QuestItemHarvestable.Name);return RunStatus.Failure;}),
                            new Action(c => { TreeRoot.StatusText = string.Format("Interacting with {0}", QuestItemHarvestable.Name); return RunStatus.Failure; }),

                            // Attack mobs in front of us if we're moving to harvest
                            new Decorator(ret => Me.IsMoving && Behaviors.Common.HostileMobInOurPath(QuestItemHarvestable.Distance + 7) != null && !Me.IsCasting && QuestItemHarvestable.Distance > QuestItemHarvestable.InteractRange * 0.85f,
                                          new Throttle(2,
                                              new Sequence(
                                                  new Action(context =>
                                                  {
                                                      Log.Info(string.Format("Hostile mob in our path, {0}, we need to kill it", Behaviors.Common.HostileMobInOurPath(QuestItemHarvestable.Distance + 5).Name));
                                                      Behaviors.Common.HostileMobInOurPath(QuestItemHarvestable.Distance + 5).Target();
                                                      BotPoi.Current = new BotPoi(Behaviors.Common.HostileMobInOurPath(QuestItemHarvestable.Distance + 5), PoiType.Kill);

                                                  })
                                                  )
                                              )),




                            // Move closer to the object
                            new Decorator(ret => !Me.Combat && QuestItemHarvestable.Distance > QuestItemHarvestable.InteractRange * 0.85f, Movement.CreateMoveToLocationBehavior(ret => QuestItemHarvestable.WorldLocation, true, ret => QuestItemHarvestable.InteractRange * 0.55f)),

                            // if we're moving then and we're in interact range then stop
                            new Decorator(ret => QuestItemHarvestable.Distance <= QuestItemHarvestable.InteractRange * 0.85f && Me.IsMoving,
                                new Sequence(
                                    Movement.CreateEnsureMovementStoppedBehavior(),

                                    // wait up to 2 seconds for us to register as stopped moving. WoW and HB tend to see this differently
                                    new Wait(2, ret => !Me.IsMoving, new ActionAlwaysSucceed())
                                    )
                                ),



                                // If we're close enough the interact with the objecb
                            new Decorator(ret => QuestItemHarvestable.Distance <= QuestItemHarvestable.InteractRange * 0.99f && !Me.IsMoving,
                                new Sequence(

                                    new Action(c => TreeRoot.StatusText = string.Format("Interacting with {0}", QuestItemHarvestable.Name)),

                                    // Finally interact with it
                                    new Action(c => { _lootableGameObject = QuestItemHarvestable; QuestItemHarvestable.Interact(); }),

                                    // wait for casting to start, max 2 seconds
                                    new Wait(2, ret => Me.IsCasting, new ActionAlwaysSucceed()),

                                    // clear all navigation as we tend to jump after we've looted
                                    new Action(c => { Navigator.Clear(); return RunStatus.Success; }),


                                    // wait while we are casting. Max 10 seconds
                                    new WaitContinue(10, ret => !Me.IsCasting, new ActionAlwaysSucceed()),

                                    // Casting has finished, but wait 2 more seconds for looting to occur
                                    new WaitContinue(1, ret => false, new ActionAlwaysSucceed()),
                                    new Wait(1, ret => Me.Looting, new ActionAlwaysSucceed()),
                                    new WaitContinue(5, ret => !Me.Looting, new ActionAlwaysSucceed()),

                                    // Black list so we don't try to loot again for min 1 minute
                                    new Action(c => Blacklist.Add(_lootableGameObject.Guid, BlacklistFlags.Loot, TimeSpan.FromMinutes(2))),
                                    new Action(c => { _lootableGameObject = null; }),

                                    // clear variable
                // clear all navigation as we tend to jump after we've looted
                                    new Action(c =>
                                    {
                                        Navigator.Clear();
                                        return RunStatus.Success;
                                    })
                                ))

                            )

                    );
        }

        private static WoWGameObject _objectToInteractWith = null;
        public static WoWGameObject QuestItemHarvestable
        {
            get
            {
                if (Setting.Instance.QuestObjectInteract.OK())
                {
                    if (_objectToInteractWith != null && _objectToInteractWith.IsValid && !Timers.AutoExpire("object to interact with", 4000)) { return _objectToInteractWith; }
                    _objectToInteractWith = QuestingAndInteraction.QuestItems(Misc.InteractRange).FirstOrDefault();

                    return _objectToInteractWith;
                }

                return null;
            }
        }
        private static WoWGameObject _lootableGameObject = null;

    }
}
