﻿using System;
using System.Linq;
using CommonBehaviors.Actions;
using LazyBoxer.Helpers;
using Styx;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.CommonBot.Routines;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;

namespace LazyBoxer.Behaviors
{
    static class Mining
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }


        public static Composite CreateOreHarvesting()
        {
            return
                new PrioritySelector(

                      // Harvest Ore
                    new Decorator(ret => Setting.Instance.InteractMining.OK() && !Misc.IsUserControllingMovement && !Me.IsCasting && !Me.IsChanneling && !Me.Combat && !Me.Looting && OreHarvestable != null,
                        new PrioritySelector(
                            new Action(c => { Log.Info("Ore found: " + OreHarvestable.Name); return RunStatus.Failure; }),
                            new Action(c => { TreeRoot.StatusText = string.Format("Mining {0}", OreHarvestable.Name); return RunStatus.Failure; }),

                            // Move closer to the object
                            new Decorator(ret => !Me.Combat && OreHarvestable.Distance > OreHarvestable.InteractRange * 0.85f, Movement.CreateMoveToLocationBehavior(ret => OreHarvestable.WorldLocation, true, ret => OreHarvestable.InteractRange * 0.55f)),

                             // Hostile mob in our path and we've flagged it for death!
                                        new Decorator(ret => Behaviors.Common.HostileMobInOurPath(OreHarvestable.Distance + 7) != null && Behaviors.Common.HostileMobInOurPath(OreHarvestable.Distance + 7).Distance < 30 && Me.GotTarget && BotPoi.Current.Type == PoiType.Kill,
                                          new Sequence(

                                              // Stop moving
                                              new DecoratorContinue(ret => Me.IsMoving, Movement.CreateEnsureMovementStoppedBehavior()),
                                              new WaitContinue(2, ret => !Me.IsMoving, new ActionAlwaysSucceed()),

                                              // Clear navigator so we don't jump
                                              new Action(context => Navigator.Clear()),

                                              // Call the CC pull logic      
                                              new Decorator(ret => RoutineManager.Current.PullBehavior != null, RoutineManager.Current.PullBehavior),

                                              // Call the CC combat logic as some CCs may not have pull logic 
                                              new Decorator(ret => RoutineManager.Current.PullBehavior != null, RoutineManager.Current.CombatBehavior)
                                          )),

                                          // Attack mobs in front of us if we're moving to harvest
                            new Decorator(ret => Me.IsMoving && Behaviors.Common.HostileMobInOurPath(OreHarvestable.Distance + 7) != null && !Me.IsCasting && OreHarvestable.Distance > OreHarvestable.InteractRange * 0.85f,
                                          new Throttle(2,
                                              new Sequence(
                                                  new Action(context =>
                                                  {
                                                      Log.Info(string.Format("Hostile mob in our path, {0}, we need to kill it", Behaviors.Common.HostileMobInOurPath(OreHarvestable.Distance + 5).Name));
                                                      Behaviors.Common.HostileMobInOurPath(OreHarvestable.Distance + 5).Target();
                                                      BotPoi.Current = new BotPoi(Behaviors.Common.HostileMobInOurPath(OreHarvestable.Distance + 5), PoiType.Kill);

                                                  })
                                                  )
                                              )),

                            // if we're moving then and we're in interact range then stop
                            new Decorator(ret => OreHarvestable.Distance <= OreHarvestable.InteractRange * 0.85f && Me.IsMoving,
                                new Sequence(
                                    Movement.CreateEnsureMovementStoppedBehavior(),

                                    // wait up to 2 seconds for us to register as stopped moving. WoW and HB tend to see this differently
                                    new Wait(2, ret => !Me.IsMoving, new ActionAlwaysSucceed())
                                    )
                                ),



                                // If we're close enough the interact with the objecb
                            new Decorator(ret => OreHarvestable.Distance <= OreHarvestable.InteractRange * 0.99f && !Me.IsMoving,
                                new Sequence(

                                    new Action(c => TreeRoot.StatusText = string.Format("Mining {0}", OreHarvestable.Name)),

                                    // Finally interact with it
                                    new Action(c => { _lootableGameObject = OreHarvestable; OreHarvestable.Interact(); }),

                                    // wait for casting to start, max 2 seconds
                                    new Wait(2, ret => Me.IsCasting, new ActionAlwaysSucceed()),

                                    // clear all navigation as we tend to jump after we've looted
                                    new Action(c => { Navigator.Clear(); return RunStatus.Success; }),

                                    // wait while we are casting. Max 10 seconds
                                    new WaitContinue(10, ret => !Me.IsCasting, new ActionAlwaysSucceed()),

                                    // Casting has finished, but wait 2 more seconds for looting to occur
                                    new WaitContinue(1, ret => false, new ActionAlwaysSucceed()),
                                    new Wait(1, ret => Me.Looting, new ActionAlwaysSucceed()),
                                    new WaitContinue(5, ret => !Me.Looting, new ActionAlwaysSucceed()),


                                    // Black list so we don't try to loot again for min 1 minute
                                    new Action(c => Blacklist.Add(_lootableGameObject.Guid, BlacklistFlags.Loot, TimeSpan.FromMinutes(2))),
                                    new Action(c => { _lootableGameObject = null; }),

                                    // clear variable
                // clear all navigation as we tend to jump after we've looted
                                    new Action(c =>
                                    {
                                        Navigator.Clear();
                                        return RunStatus.Success;
                                    })
                                ))

                            )
                        )




                    );
        }

        private static WoWGameObject _oreToMine = null;
        public static WoWGameObject OreHarvestable
        {
            get
            {
                if (Setting.Instance.InteractMining.OK())
                {
                    if (Units.IsObjectValid<WoWGameObject>(_oreToMine)) return _oreToMine;
                    //if (_oreToMine != null && _oreToMine.IsValid && !Timers.AutoExpire("Ore to harvest", 4000)) { return _oreToMine; }
                    //_oreToMine = QuestingAndInteraction.OreItems(Misc.InteractRange).FirstOrDefault();

                    _oreToMine = ObjectManager.GetObjectsOfTypeFast<WoWGameObject>()
                                              .Where( o =>
                                                  o.IsMineral && o.CanLoot &&
                                                  !Misc.IsPlayerCompeting(o) &&
                                                  !Blacklist.Contains(o.Guid, BlacklistFlags.Node) &&
                                                  o.WorldLocation.Distance(StyxWoW.Me.Location) <= Misc.InteractRange)
                                              .OrderBy(o => o.Distance).FirstOrDefault();

                    return _oreToMine;
                }

                return null;
            }
        }

        private static WoWGameObject _lootableGameObject = null;


    }
}
