﻿using CommonBehaviors.Actions;
using LazyBoxer.Helpers;
using Styx;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;

namespace LazyBoxer.Behaviors
{
    static class MoveAndFaceTarget
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }

        public static Composite CreateCombatMovementAndFaceTarget()
        {
            return

                new PrioritySelector(

                    // Face target
                    new Decorator(ret => Setting.Instance.FaceTarget.OK() && !Misc.IsUserControllingMovement && !Me.IsMoving && !Me.IsSafelyFacing(CT, 85),
                        new Sequence(
                            new Action(context =>  CT.Face())
                            )),

                    // Face target. Trying to ensure it faces faster when moving behind the target
                    new Decorator(ret => Setting.Instance.FaceTarget.OK() && !Misc.IsUserControllingMovement && Setting.Instance.AttackFromBehind.OK() && CT.CurrentTargetGuid != Me.Guid && Me.IsSafelyBehind(CT),
                        new Sequence(
                            new Action(context => CT.Face()),
                            new ActionAlwaysFail()
                            )),

                    // Combat movement
                    new Decorator(ret => Setting.Instance.MoveToTarget.OK() && !Misc.IsUserControllingMovement && !Setting.Instance.CombatFollow.OK(),
                        new PrioritySelector(

                            // Casters, move to LOS
                            new Decorator(ret=> !Extensions.IsMeMeleeClass && Me.GotTarget && !CT.InLineOfSpellSight,
                                          new PrioritySelector(
                                              Movement.CreateMoveToLosBehavior(ret=> CT)
                                              )),

                            // Position character behind the mob
                            new Decorator(ret => Setting.Instance.AttackFromBehind.OK() && CT.CurrentTargetGuid != Me.Guid && !Me.IsSafelyBehind(CT),
                                            new PrioritySelector(
                                                Movement.CreateMoveBehindTargetBehavior()
                                                )
                                ),

                            // Melee movement
                            new Decorator(ret => Extensions.IsMeMeleeClass,
                                new PrioritySelector(
                                    Movement.CreateEnsureMovementStoppedWithinMelee(),
                                    Movement.CreateMoveToMeleeBehavior(true)
                                )),

                            // Caster movement
                            new Decorator(ret => !Extensions.IsMeMeleeClass && Me.GotTarget,
                                new PrioritySelector(

                                    // LOS is all important
                                    //Movement.CreateMoveToLosBehavior(),

                                    // If we're in melee range then REALLY stop moving
                                    Movement.CreateEnsureMovementStoppedWithinMelee(),

                                    // Move into range
                                    new Decorator(ret => CT.Distance >= Setting.Instance.MoveToTargetDistance,
                                        new PrioritySelector(
                                            Movement.CreateMoveToTargetBehavior(true, (Setting.Instance.MoveToTargetDistance * (float)0.85))
                                            )),

                                    // Stop moving
                                    new Decorator(ret => CT.Distance <= Setting.Instance.MoveToTargetDistance * 0.90 && Me.IsMoving && CT.InLineOfSpellSight,
                                        new PrioritySelector(Movement.CreateEnsureMovementStoppedBehavior())
                                        )
                                ))
                        )
                    )





                );
        }


    }
}
