﻿using System;
using System.Linq;
using CommonBehaviors.Actions;
using LazyBoxer.Helpers;
using JetBrains.Annotations;
using Styx;
using Styx.CommonBot;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;

namespace LazyBoxer.Behaviors
{
    static class Questing
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }


        
        public static Composite CreateQuestPickupOrTurninBehavior()
        {
            return new Sequence(
                new Decorator(ret => TargetIsInteractable, new ActionAlwaysSucceed()),
                new Action(c => { if (FollowTheLeader.Leader != null) FollowTheLeader.Leader.CurrentTarget.Target(); }),
                new WaitContinue(2, ret => FollowTheLeader.Leader != null && StyxWoW.Me.CurrentTarget == FollowTheLeader.Leader.CurrentTarget || Setting.Instance.FollowLeader.Contains("None"), new ActionAlwaysSucceed()),
                new Action(c =>
                {
                    if (Me.CurrentTarget.WithinInteractRange) return RunStatus.Success;

                    Navigator.MoveTo(Me.CurrentTarget.WorldLocation);
                    return RunStatus.Running;
                }
                    ),
                new Action(c => Navigator.PlayerMover.MoveStop()),
                new Action(c => Me.CurrentTarget.Interact()),
                new Action(c => Navigator.Clear()),

                //Most of the code here is done through events (Up above)
                new Wait(2, ret => false, new ActionAlwaysSucceed()),
                new Action(c => Navigator.Clear())
                );



        }

        public static Composite CreateQuestFetch()
        {
            return new Sequence(
                new Decorator(ret => InteractableNPC != null, new ActionAlwaysSucceed()),
                new Action(c => InteractableNPC.Target()),
                new WaitContinue(2, ret => StyxWoW.Me.CurrentTarget == InteractableNPC, new ActionAlwaysSucceed()),
                new Action(c =>
                {
                    if (Me.CurrentTarget.WithinInteractRange) return RunStatus.Success;

                    Navigator.MoveTo(Me.CurrentTarget.WorldLocation);
                    return RunStatus.Running;
                }
                    ),
                new Action(c => Navigator.PlayerMover.MoveStop()),
                new Action(c => Me.CurrentTarget.Interact()),

                //Most of the code here is done through events (Up above)
                new Wait(3, ret => false, new ActionAlwaysSucceed()),
                new Action(c => Navigator.Clear())
                );



        }

        private static WoWUnit InteractableNPC
        {
            get
            {
                WoWUnit npc = ObjectManager.GetObjectsOfTypeFast<WoWUnit>().Where(target => (
                    target.QuestGiverStatus == QuestGiverStatus.TurnIn ||
                    target.QuestGiverStatus == QuestGiverStatus.Available ||
                    target.QuestGiverStatus == QuestGiverStatus.AvailableRepeatable ||
                    target.QuestGiverStatus == QuestGiverStatus.TurnInRepeatable

                    )).OrderBy(d => d.Distance).FirstOrDefault();


                return npc;

            }
        }

        private static bool TargetIsInteractable
        {
            get
            {
                WoWUnit target = null;
                if (Setting.Instance.FollowLeader.Contains("None"))
                {
                    if (Me.GotTarget) target = Me.CurrentTarget;
                }
                else if (FollowTheLeader.Leader != null && FollowTheLeader.Leader.GotTarget)
                {
                    target = FollowTheLeader.Leader.CurrentTarget;
                }

                if (target == null) return false;

                if (target.QuestGiverStatus == QuestGiverStatus.TurnIn ||
                    target.QuestGiverStatus == QuestGiverStatus.Available ||
                    target.QuestGiverStatus == QuestGiverStatus.AvailableRepeatable ||
                    target.QuestGiverStatus == QuestGiverStatus.TurnInRepeatable ||
                    //target.IsFlightMaster ||
                    //target.IsBanker ||
                    //target.IsInnkeeper ||
                    target.InteractType == WoWInteractType.TaxiPathAvailable)
                    return true;




                return false;
            }
        }
    }
}
