﻿using System;
using System.Linq;
using CommonBehaviors.Actions;
using LazyBoxer.Helpers;
using Styx;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.CommonBot.Routines;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;

namespace LazyBoxer.Behaviors
{
    static class Skinning
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }

        private static int _attemptCount = 0;
        private static WoWUnit _tempSkinnable = null;

        public static Composite SkinTest()
        {
            return new PrioritySelector(
                
                new Decorator(ret => Setting.Instance.InteractSkinning.OK() && !Misc.IsUserControllingMovement && !Me.IsCasting && !Me.IsChanneling && !Me.Combat && !Me.Looting && SkinnableMob != null,
                    new Action(c=> BotPoi.Current = new BotPoi(SkinnableMob, PoiType.Skin))
                
                ));
        }

        public static Composite CreateSkinning()
        {
            return
                new PrioritySelector(
                // Harvest Herbs
                    new Decorator(ret => Setting.Instance.InteractSkinning.OK() && !Misc.IsUserControllingMovement && !Me.IsCasting && !Me.IsChanneling && !Me.Combat && !Me.Looting && SkinnableMob != null,
                        new PrioritySelector(

                            new Action(c => { Log.Info("Skinnable mob found: " + SkinnableMob.Name); return RunStatus.Failure; }),
                            new Action(c => { TreeRoot.StatusText = string.Format("Skinning {0}", SkinnableMob.Name); return RunStatus.Failure; }),
                            //new Action(c => { BotPoi.Current = new BotPoi(SkinnableMob, PoiType.Skin); return RunStatus.Failure; }),

                            // Move closer to the object
                            new Decorator(ret => !Me.Combat && SkinnableMob.Distance > SkinnableMob.InteractRange * 0.85f, Movement.CreateMoveToLocationBehavior(ret => SkinnableMob.WorldLocation, true, ret => SkinnableMob.InteractRange * 0.55f)),

                            // Hostile mob in our path and we've flagged it for death!
                            new Decorator(ret => Common.HostileMobInOurPath(SkinnableMob.Distance + 7) != null && Common.HostileMobInOurPath(SkinnableMob.Distance + 7).Distance < 30 && Me.GotTarget && BotPoi.Current.Type == PoiType.Kill,
                                new Sequence(

                                    // Stop moving
                                    new DecoratorContinue(ret => Me.IsMoving, Movement.CreateEnsureMovementStoppedBehavior()),
                                    new WaitContinue(2, ret => !Me.IsMoving, new ActionAlwaysSucceed()),

                                    // Clear navigator so we don't jump
                                    new Action(context => Navigator.Clear()),

                                    // Call the CC pull logic      
                                    new Decorator(ret => RoutineManager.Current.PullBehavior != null, RoutineManager.Current.PullBehavior),

                                    // Call the CC combat logic as some CCs may not have pull logic 
                                    new Decorator(ret => RoutineManager.Current.PullBehavior != null, RoutineManager.Current.CombatBehavior)
                                    )),

                            // Attack mobs in front of us if we're moving to harvest
                            new Decorator(ret => Me.IsMoving && Common.HostileMobInOurPath(SkinnableMob.Distance + 7) != null && !Me.IsCasting && SkinnableMob.Distance > SkinnableMob.InteractRange * 0.85f,
                                new Throttle(2,
                                             new Sequence(
                                                 new Action(context =>
                                                 {
                                                     Log.Info(string.Format("Hostile mob in our path, {0}, we need to kill it", Common.HostileMobInOurPath(SkinnableMob.Distance + 5).Name));
                                                     Common.HostileMobInOurPath(SkinnableMob.Distance + 5).Target();
                                                     BotPoi.Current = new BotPoi(Common.HostileMobInOurPath(SkinnableMob.Distance + 5), PoiType.Kill);

                                                 })
                                                 )
                                    )),

                            // if we're moving then and we're in interact range then stop
                            new Decorator(ret => SkinnableMob.Distance <= SkinnableMob.InteractRange * 0.85f && Me.IsMoving,
                                new Sequence(
                                    Movement.CreateEnsureMovementStoppedBehavior(),

                                    // wait up to 2 seconds for us to register as stopped moving. WoW and HB tend to see this differently
                                    new Wait(2, ret => !Me.IsMoving, new ActionAlwaysSucceed())
                                    )
                                ),



                            // If we're close enough the interact with the objecb
                            new Decorator(ret => SkinnableMob.Distance <= SkinnableMob.InteractRange * 0.99f && !Me.IsMoving,
                                new Sequence(

                                    new Action(c => TreeRoot.StatusText = string.Format("Skinning {0}", SkinnableMob.Name)),

                                    // Too many failed attempts?
                                    new DecoratorContinue(ret => _attemptCount > 3,
                                        new Sequence(
                                            new Action(context => Log.Info(string.Format("Too many failed attempts at skinning {0}, black listing it!", SkinnableMob.Name))),
                                            new Action(c => _attemptCount = 0),
                                            new Action(c => Blacklist.Add(SkinnableMob.Guid, BlacklistFlags.Interact, TimeSpan.FromMinutes(10)))
                                            )),

                                    // Finally interact with it
                                    new Action(c => { _tempSkinnable = SkinnableMob;
                                                        try { SkinnableMob.Interact(); } catch (Exception e) {  }
                                                        _attemptCount += 1;
                                    }),

                                    // wait for casting to start, max 2 seconds
                                    new Wait(1, ret => Me.IsCasting, new ActionAlwaysSucceed()),

                                    // clear all navigation as we tend to jump after we've looted
                                    new Action(c => { Navigator.Clear(); return RunStatus.Success; }),

                                    // wait while we are casting. Max 10 seconds
                                    new WaitContinue(10, ret => !Me.IsCasting, new ActionAlwaysSucceed()),

                                    new Action(c => { if (Me.Looting) Blacklist.Add(_tempSkinnable.Guid, BlacklistFlags.Interact, TimeSpan.FromMinutes(10)); }),

                                    // Casting has finished, but wait 2 more seconds for looting to occur
                                    new WaitContinue(1, ret => false, new ActionAlwaysSucceed()),

                                    // If we're looting it skinning was successful so blacklist it so we don't try again.
                                    new Action(c => { if (Me.Looting) Blacklist.Add(_tempSkinnable.Guid, BlacklistFlags.Interact, TimeSpan.FromMinutes(10)); }),
                                    new Wait(1, ret => Me.Looting, new ActionAlwaysSucceed()),
                                    new WaitContinue(5, ret => !Me.Looting, new ActionAlwaysSucceed()),

                                    // Black list so we don't try to loot again for min 1 minute
                                    new Action(c => Blacklist.Add(_tempSkinnable.Guid, BlacklistFlags.Interact, TimeSpan.FromMinutes(10))),
                                    new Action(c => { _tempSkinnable = null; _attemptCount = 0; }),

                                    // clear variable
                                    // clear all navigation as we tend to jump after we've looted
                                    new Action(c => { Navigator.Clear(); return RunStatus.Success; })
                                    ))

                            )
                        )




                    );
        }

        private static WoWUnit _mobtoSkin = null;
        public static WoWUnit SkinnableMob
        {
            get
            {
                
                if (Setting.Instance.InteractSkinning.OK())
                {
                    if (Units.IsObjectValid<WoWUnit>(_mobtoSkin)) return _mobtoSkin;

                    _mobtoSkin = ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                                     .Where(o =>
                                         o.Distance < Misc.InteractRange && o.Skinnable && 
                                         //o.CanSkin &&
                                         !Misc.IsPlayerCompeting(o) &&
                                         !Blacklist.Contains(o.Guid, BlacklistFlags.Interact))
                                     .OrderBy(o => o.Distance)
                                     .FirstOrDefault();

                    return _mobtoSkin;
                }

                return null;
            }
        }

    }
}
