﻿using System;
using System.Linq;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Styx;
using Styx.CommonBot;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace LazyBoxer.Helpers
{
    static class Misc
    {

        public static int InteractRange
        {
            get
            {
                return StyxWoW.Me.IsInInstance || StyxWoW.Me.GroupInfo.IsInRaid
                           ? Setting.Instance.InteractRangeInstance
                           : Setting.Instance.InteractRangeWorld;
            }
        }

        /// <summary>
        /// If a player is near this object then blacklist it for a few seconds. Assumes they are going to harvest it
        /// </summary>
        /// <param name="target"></param>
        /// <param name="distanceFromTarget"></param>
        /// <param name="blacklistSeconds"></param>
        /// <returns></returns>
        public static bool IsPlayerCompeting(WoWObject target, int distanceFromTarget = 10, int blacklistSeconds = 10)
        {
            try
            {

                

                WoWUnit nearestPlayerToObject = null;

                using (StyxWoW.Memory.AcquireFrame())
                {
                    nearestPlayerToObject = ObjectManager.GetObjectsOfTypeFast<WoWPlayer>()
                                                         .OrderBy(p => p.Location.Distance(target.Location))
                                                         .FirstOrDefault(p => p.IsPlayer && p.IsAlive && !p.IsMe && !p.IsInOurParty());
                }

                // If player is too close to the target ignore it for a bit...
                if (nearestPlayerToObject != null && (nearestPlayerToObject.Location.Distance(target.Location) <= distanceFromTarget))
                {
                    Blacklist.Add(target, BlacklistFlags.All, TimeSpan.FromSeconds(blacklistSeconds));
                    return (true);
                }
            }

            catch (Exception e) {  }

            return (false);
        }

        public static bool IsUserControllingMovement
        {
            get
            {

                /*
                if (KeyboardPolling.IsKeyDown(Keys.Delete))
                {
                    Log.Info("======== DELETE KEY PRESSED");
                }
                 */
                // if the current WoW window does not have focus then ignore the [global] keydown overrides
                if (StyxWoW.Memory.Process.Id != KeyboardPolling.ForegroundWindowPID) return false;

                if (KeyboardPolling.IsKeyDown(Keys.W)) return true;
                if (KeyboardPolling.IsKeyDown(Keys.S)) return true;
                if (KeyboardPolling.IsKeyDown(Keys.A)) return true;
                if (KeyboardPolling.IsKeyDown(Keys.D)) return true;
                if (KeyboardPolling.IsKeyDown(Keys.Q)) return true;
                if (KeyboardPolling.IsKeyDown(Keys.E)) return true;

                if (KeyboardPolling.IsKeyDown(Keys.Left)) return true;
                if (KeyboardPolling.IsKeyDown(Keys.Right)) return true;
                if (KeyboardPolling.IsKeyDown(Keys.Up)) return true;
                if (KeyboardPolling.IsKeyDown(Keys.Down)) return true;

                if (KeyboardPolling.IsKeyDown(Keys.LButton) && KeyboardPolling.IsKeyDown(Keys.RButton)) return true;

                return false;
            }
        }
    }


   
    

    static class KeyboardPolling
    {
        [DllImport("user32.dll")]
        private static extern short GetAsyncKeyState(Keys vKey);
        
        public static bool IsKeyDown(Keys key)
        {
            return (GetAsyncKeyState(key)) != 0;
        }

        public static bool IsKeyUp(Keys key)
        {
            return (GetAsyncKeyState(key)) == 0;
        }
        
        [DllImport("user32.dll")]
        public static extern IntPtr GetForegroundWindow();

        [DllImport("user32.dll", SetLastError = true)]
        public static extern int GetWindowThreadProcessId(IntPtr hWnd, out int lpdwProcessId);

        public static int ForegroundWindowPID
        {
            get
            {
                int processID = 0;
                int threadID = GetWindowThreadProcessId(GetForegroundWindow(), out processID);
                return processID;
            }
        }

    }
}
